"""
Common portfolio metrics
"""

import numpy as np
import pandas as pd


def sharpe_ratio_daily(returns, risk_free_rate=0):
    """
    Compute the Sharpe ratio of a strategy.

    Assumes index is a datetime of days.

    Not sure how NaNs are handled.
    """
    retseries = returns.values
    return np.sqrt(365) * (np.mean(retseries) - risk_free_rate) / np.std(retseries)


def sortino_ratio_daily(returns, risk_free_rate=0):
    """
    Compute the Sortino ratio of a strategy.

    Assumes index is a datetime of days.

    Also not sure how NaNs are handled.
    """
    retseries = returns.values

    if len(retseries[retseries < 0]) > 1:
        downside_returns = retseries[retseries < 0]
    else:
        return np.inf
    return (
        np.sqrt(365) * (np.mean(retseries) - risk_free_rate) / np.std(downside_returns)
    )


def calmar_ratio_daily(returns, risk_free_rate=0):
    """
    Compute the Calmar ratio of a strategy.

    Assumes index is a datetime of days.
    """
    retseries = returns.values

    if retseries.min() < 0:
        drawdown = abs(retseries.min())
    else:
        return np.inf
    return annualized_return(returns) / drawdown


def annualized_return(returns, days_in_year=365):
    """
    Compute the annualized return of a strategy.

    Assumes index is a datetime of days.
    """
    period = returns.shape[0]
    retseries = returns.values
    return np.power(np.e, np.log(1 + retseries).sum() * days_in_year / period) - 1


def cumulative_return(returns):
    """
    Compute the cumulative return of a strategy.

    Assumes index is a datetime of days.
    """
    retseries = returns.values
    return (1 + retseries).cumprod()[-1] - 1


def max_drawdown(returns):
    """
    Compute the max drawdown of a strategy.

    Assumes index is a datetime of days.
    """
    retseries = returns.values

    return abs(retseries.min()) if retseries.min() < 0 else 0


def annualized_volatility(returns, days_in_year=365):
    """
    Compute the daily volatility of a strategy.

    Assumes index is a datetime of days.
    """
    retseries = returns.values
    return retseries.std() * np.sqrt(days_in_year) * 100


def value_at_risk(returns, percentile=5):
    """
    Compute the VaR of a strategy.
    """
    retseries = returns.values
    return np.percentile(retseries, percentile)


def expected_shortfall(returns, percentile=5):
    """
    Compute the expected_shortfall of a strategy.
    """
    retseries = returns.values

    thresh = np.percentile(retseries, percentile)
    return retseries[retseries < thresh].mean()


def tear_sheet(returns, format=False):
    """
    Generate a tear sheet of a strategy.
    """
    df = pd.DataFrame()
    df["Annualized Return"] = annualized_return(returns)
    df["Cumulative Return"] = cumulative_return(returns)
    df["Sharpe"] = sharpe_ratio_daily(returns)
    df["Sortino"] = sortino_ratio_daily(returns)
    df["Calmar"] = calmar_ratio_daily(returns)
    df["Max Drawdown"] = max_drawdown(returns)
    df["Annualized Volatility"] = annualized_volatility(returns)
    df["Value at Risk"] = value_at_risk(returns)
    df["Expected Shortfall"] = expected_shortfall(returns)
    df = pd.DataFrame(
        (
            annualized_return(returns),
            cumulative_return(returns),
            sharpe_ratio_daily(returns),
            sortino_ratio_daily(returns),
            calmar_ratio_daily(returns),
            max_drawdown(returns),
            annualized_volatility(returns),
            value_at_risk(returns),
            expected_shortfall(returns),
        ),
        index=[
            "Annualized Return",
            "Cumulative Return",
            "Sharpe",
            "Sortino",
            "Calmar",
            "Max Drawdown",
            "Annualized Volatility",
            "Value at Risk",
            "Expected Shortfall",
        ],
    ).T

    if format:
        df["Annualized Return"] = ((df["Annualized Return"] * 100).round(2)).astype(
            "str"
        ) + "%"
        df["Cumulative Return"] = ((df["Cumulative Return"] * 100).round(2)).astype(
            "str"
        ) + "%"
        df["Sharpe"] = df["Sharpe"].round(2)
        df["Sortino"] = df["Sortino"].round(2)
        df["Calmar"] = df["Calmar"].round(2)
        df["Max Drawdown"] = ((df["Max Drawdown"] * 100).round(2)).astype("str") + "%"
        df["Annualized Volatility"] = ((df["Annualized Volatility"]).round(2)).astype(
            "str"
        ) + "%"
        df["Value at Risk"] = (df["Value at Risk"] * 100).round(2).astype("str") + "%"
        df["Expected Shortfall"] = (df["Expected Shortfall"] * 100).round(2).astype(
            "str"
        ) + "%"

    return df



import pytest
import pandas as pd
from datetime import datetime
import numpy as np

from friktionutils.metrics import (
    sharpe_ratio_daily,
    sortino_ratio_daily,
    calmar_ratio_daily,
    annualized_return,
    cumulative_return,
    max_drawdown,
    annualized_volatility,
    value_at_risk,
    expected_shortfall,
    tear_sheet,
)
from friktionutils.options import random_walk

df = pd.DataFrame()
df.index = pd.date_range(start="1/1/2020", periods=700, freq="D")
df["returns"] = random_walk(1, 700, 0, 100)
df["returns"] = df.returns - 1
print(df)


@pytest.mark.parametrize(
    "returns",
    [
        (df),
    ],
)
def test_sharpe_ratio_daily(returns):
    print(tear_sheet(df.returns))
    assert False
    # assert np.mean(df.returns) == df.mean().iloc[0]
    # assert x == (df.mean() / df.std() * np.sqrt(700)).iloc[0]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['friktionutils']

package_data = \
{'': ['*'], 'friktionutils': ['.pytest_cache/*', '.pytest_cache/v/cache/*']}

install_requires = \
['numpy>=1.22.3,<2.0.0',
 'pandas>=1.4.2,<2.0.0',
 'pytest>=7.1.2,<8.0.0',
 'requests>=2.27.1,<3.0.0']

setup_kwargs = {
    'name': 'friktion-research-utils',
    'version': '0.0.39',
    'description': 'Friktion research toolkit',
    'long_description': '',
    'author': 'thiccythot',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://app.friktion.fi',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



import pytest
import pandas as pd
from datetime import datetime
import matplotlib.pyplot as plt

# from utils import options

from friktionutils.options import Surface, Curve, find_strike

df = pd.read_csv("btc_call.csv")
df2 = pd.read_csv("btc_put.csv")
df3 = pd.read_csv("sol_put.csv")

CALL = Surface(df, datetime(2022, 1, 1), "C", "BTC", 20000)
PUT = Surface(df2, datetime(2022, 1, 1), "P", "BTC", 20000)
SOLPUT = Surface(df3, datetime(2022, 7, 18), "P", "SOL", 40)


@pytest.mark.parametrize(
    "strike, delta",
    [
        (
            1.0,
            0.1,
        ),
    ],
)
def test_getIvFromStrike(strike, delta):
    iv = SOLPUT.curves[0].getIvFromDelta(delta)
    strike = find_strike("P", SOLPUT.spot, 7 / 365, 0, iv / 100, delta)
    print(SOLPUT.spot, strike, delta, iv)
    assert iv


@pytest.mark.parametrize(
    "delta",
    [
        (0.1),
    ],
)
def test_getIvFromDelta_sol_put(delta):
    iv = SOLPUT.curves[0].getIvFromDelta(delta)
    print(SOLPUT.symbol)
    assert iv == 143.24369863013698
    print(delta, iv)


@pytest.mark.parametrize(
    "strike",
    [
        (100),
    ],
)
def test_getIvFromStrike_low_call(strike):
    iv = CALL.curves[0].getIvFromStrike(strike)
    assert iv == 67.6
    print(strike, iv)


@pytest.mark.parametrize(
    "strike",
    [
        (25000),
    ],
)
def test_getIvFromStrike_mid_call(strike):
    iv = CALL.curves[0].getIvFromStrike(strike)
    assert iv == 70.4
    print(strike, iv)


@pytest.mark.parametrize(
    "strike",
    [
        (250000),
    ],
)
def test_getIvFromStrike_mid_high(strike):
    iv = CALL.curves[0].getIvFromStrike(strike)
    assert iv == 137.9
    print(strike, iv)


@pytest.mark.parametrize(
    "strike",
    [
        (25200),
    ],
)
def test_getIvFromStrike_mid_put(strike):
    iv = PUT.curves[0].getIvFromStrike(strike)
    assert iv == 75.036
    print(strike, iv)


def test_find_strike():
    print(PUT.curves[0].getIvFromStrike(20000))
    print(PUT.curves[0].getIvFromDelta(0.4))
    print()
    assert False
